"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CKBNodeBase = void 0;
const lumos_1 = require("@ckb-lumos/lumos");
const node_fs_1 = __importDefault(require("node:fs"));
const kuai_common_1 = require("@ckb-js/kuai-common");
const node_path_1 = __importDefault(require("node:path"));
class CKBNodeBase {
    async doDeploy(rpc, indexer, from, privateKey, script, filePath, cellDeps, depType = 'code') {
        const scriptBinary = node_fs_1.default.readFileSync(filePath);
        let txSkeleton = (await lumos_1.commons.deploy.generateDeployWithDataTx({
            cellProvider: indexer,
            scriptBinary,
            fromInfo: from,
        })).txSkeleton;
        cellDeps === null || cellDeps === void 0 ? void 0 : cellDeps.forEach((dep) => {
            txSkeleton = txSkeleton.update('cellDeps', (cellDeps) => cellDeps.push(dep.cellDep));
        });
        const txHash = await rpc.sendTransaction(this.sign(txSkeleton, privateKey));
        try {
            await (0, kuai_common_1.waitUntilCommitted)(rpc, txHash);
        }
        catch (e) {
            console.error(e);
        }
        return {
            name: script,
            path: filePath,
            depType,
            cellDeps,
            outPoint: {
                txHash,
                index: '0x0',
            },
        };
    }
    async deployBuiltInScripts(scriptName, builtInDirPath, indexer, rpc, from, privateKey) {
        const scripts = [];
        for (const script of scriptName) {
            const filePath = node_path_1.default.join(builtInDirPath, script);
            scripts.push(await this.doDeploy(rpc, indexer, from, privateKey, script, filePath));
        }
        return scripts;
    }
    sign(txSkeleton, privateKey) {
        txSkeleton = lumos_1.commons.common.prepareSigningEntries(txSkeleton);
        const signature = lumos_1.hd.key.signRecoverable(txSkeleton.get('signingEntries').get(0).message, privateKey);
        return lumos_1.helpers.sealTransaction(txSkeleton, [signature]);
    }
    async deployCustomScripts(filePath, indexer, rpc, from, privateKey) {
        const scripts = [];
        if (node_fs_1.default.existsSync(filePath)) {
            const configs = JSON.parse(node_fs_1.default.readFileSync(filePath).toString());
            if (configs.custom && Array.isArray(configs.custom)) {
                for (const script of configs.custom) {
                    const deployedScript = await this.doDeploy(rpc, indexer, from, privateKey, script.name, script.path, script.cellDeps
                        ? scripts
                            .filter((v) => { var _a; return (_a = script.cellDeps) === null || _a === void 0 ? void 0 : _a.map((v) => v.name).includes(v.name); })
                            .map((v) => {
                            return { name: v.name, cellDep: { depType: v.depType, outPoint: v.outPoint } };
                        })
                        : undefined, script.depType);
                    scripts.push(deployedScript);
                }
            }
        }
        return scripts;
    }
    async deployScripts({ builtInScriptName, configFilePath, builtInDirPath, indexer, rpc, privateKey, }) {
        const from = lumos_1.helpers.encodeToConfigAddress(lumos_1.hd.key.privateKeyToBlake160(privateKey), 'SECP256K1_BLAKE160');
        await indexer.waitForSync();
        const config = {
            builtIn: await this.deployBuiltInScripts(builtInScriptName, builtInDirPath, indexer, rpc, from, privateKey),
            custom: await this.deployCustomScripts(configFilePath, indexer, rpc, from, privateKey),
        };
        node_fs_1.default.writeFileSync(configFilePath, Buffer.from(JSON.stringify(config)), { flag: 'w' });
    }
    async generateLumosConfig() {
        const rpc = new lumos_1.RPC(this.url);
        const block = await rpc.getBlockByNumber('0x0');
        this.lumosConfig = {
            PREFIX: 'ckt',
            SCRIPTS: {
                SECP256K1_BLAKE160: {
                    CODE_HASH: '0x9bd7e06f3ecf4be0f2fcd2188b23f1b9fcc88e5d4b65a8637b17723bbda3cce8',
                    HASH_TYPE: 'type',
                    TX_HASH: block.transactions[1].hash,
                    INDEX: '0x0',
                    DEP_TYPE: 'depGroup',
                    SHORT_ID: 0,
                },
                SECP256K1_BLAKE160_MULTISIG: {
                    CODE_HASH: '0x5c5069eb0857efc65e1bca0c07df34c31663b3622fd3876c876320fc9634e2a8',
                    HASH_TYPE: 'type',
                    TX_HASH: block.transactions[1].hash,
                    INDEX: '0x1',
                    DEP_TYPE: 'depGroup',
                    SHORT_ID: 1,
                },
            },
        };
    }
}
exports.CKBNodeBase = CKBNodeBase;
